<?php

namespace QInstruments\ProductSystem\DataProcessor;

use QInstruments\ProductSystem\Domain\Repository\ProductRepository;
use TYPO3\CMS\Frontend\ContentObject\ContentObjectRenderer;
use TYPO3\CMS\Frontend\ContentObject\DataProcessorInterface;

class ProductConfiguratorDataProcessor implements DataProcessorInterface
{
    public function __construct(protected readonly ProductRepository $productRepository) {}

    protected function getCheckboxOptions(array $fieldConfig): array
    {
        $options = [];
        foreach ($fieldConfig['config']['items'] as $index => $optionConfig) {
            $indicatorValue = 2 ** $index;
            $options[] = [
                'index' => $index,
                'indicatorValue' => $indicatorValue,
                'label' => $optionConfig['label'],
                'value' => $indicatorValue,
            ];
        }
        return $options;
    }

    protected function getOptionLabels(int $intValue, array $options)
    {
        $labels = [];
        foreach ($options as $option) {
            if ($intValue & $option['value']) {
                $labels[] = $option['label'];
            }
        }
        return $labels;
    }

    public function process(
        ContentObjectRenderer $cObj,
        array $contentObjectConfiguration,
        array $processorConfiguration,
        array $processedData
    ) {

        $tcaModel = $GLOBALS['TCA']['tx_productsystem_domain_model_product'];
        $areaOptions = $this->getCheckboxOptions($tcaModel['columns']['areas']);
        $labwareOptions = $this->getCheckboxOptions($tcaModel['columns']['labwares']);
        $methodOptions = $this->getCheckboxOptions($tcaModel['columns']['methods']);

        $processedData['allAreas'] = array_map(fn($opt) => $opt['label'], $areaOptions);
        $processedData['allLabwares'] = array_map(fn($opt) => $opt['label'], $labwareOptions);
        $processedData['allMethods'] = array_map(fn($opt) => $opt['label'], $methodOptions);

        $productRows = $this->productRepository->findAll()->toArray();
        $results = [];
        foreach ($productRows as $row) {
            $result = $row->toArray();
            $result['areas'] = $this->getOptionLabels($result['areas'], $areaOptions);
            $result['labwares'] = $this->getOptionLabels($result['labwares'], $labwareOptions);
            $result['methods'] = $this->getOptionLabels($result['methods'], $methodOptions);
            $results[] = $result;
        }

        $processedData['availableProducts'] = $results;

        return $processedData;
    }
}
