import { parse, type Options } from 'csv-parse/browser/esm'
import { z } from 'zod'

import { EmptyOrFilled, Labware, Method, UsageArea, type Product } from '@/models'

import PRODUCTS_CSV from './products.csv?raw'

const ProductCsvRow = z.object({
  Name: z.string(),
  PartNo: z.string(),
  BgColor: z.string(),
  FgColor: z.string(),
  UsageArea: UsageArea,
  Shake: EmptyOrFilled,
  Heat: EmptyOrFilled,
  Cool: EmptyOrFilled,
  Tilt: EmptyOrFilled,
  Turn: EmptyOrFilled,
  TubesVialsLarge: EmptyOrFilled,
  TubesVialsSmall: EmptyOrFilled,
  Microplates: EmptyOrFilled,
  MicroplatesHighDensity: EmptyOrFilled,
  MicroplatesMediumDensity: EmptyOrFilled,
  MicroplatesLowDensity: EmptyOrFilled,
  DeepWellPlates: EmptyOrFilled,
  AdapterNeeded: EmptyOrFilled,
  MagneticBeadsNeeded: EmptyOrFilled,
})
type ProductCsvRow = z.infer<typeof ProductCsvRow>

const getRowAreas = (row: ProductCsvRow): UsageArea[] => [row.UsageArea]
const getRowLabwares = (row: ProductCsvRow): Labware[] =>
  Labware._def.values.filter((key) => row[key])
const getRowMethods = (row: ProductCsvRow): Method[] => Method._def.values.filter((key) => row[key])

const mapProductRow = (row: ProductCsvRow): Product => ({
  name: row.Name,
  partNo: row.PartNo,
  fgColor: row.FgColor,
  bgColor: row.BgColor,
  areas: getRowAreas(row),
  labwares: getRowLabwares(row),
  methods: getRowMethods(row),
  adapterRequired: row.AdapterNeeded,
  magneticBeadsRequired: row.MagneticBeadsNeeded,
  techSpecs: [],
})

const parseCsvRows = async <T>(text: string, opts?: Options): Promise<T[]> => {
  return await new Promise<T[]>((resolve, reject) => {
    parse(text, opts, (err, records) => {
      if (err) return reject(err)
      else return resolve(records)
    })
  })
}
export const loadProducts = async (): Promise<Product[]> => {
  const rows = await parseCsvRows<ProductCsvRow>(PRODUCTS_CSV, {
    columns: true,
    skip_empty_lines: true,
    delimiter: '\t',
  })
  return rows.map((row) => ProductCsvRow.parse(row)).map(mapProductRow)
}
